@@ -1,5 +1,6 @@
 import re
 from urllib.request import urlopen
+from ignorem import config
 
 _REPO_LIST = [
 	"https://raw.githubusercontent.com/itsmaxymoo/ignorem/main/SOURCES"
@@ -8,9 +9,18 @@ _got_sources = False
 _sources = []
 
 
+def get_external_repos():
+	external_repo = config.get("repos")
+	if not isinstance(external_repo, list):
+		config.store("repos", [])
+
+	return external_repo
+
+
 def _get_all_sources():
 	global _got_sources
 	global _sources
+	global _REPO_LIST
 
 	# Don't fetch sources if already did once per-program-execution
 	if _got_sources:
@@ -19,9 +29,12 @@ def _get_all_sources():
 	big_list = ""
 
 	# Download each repo
-	for r in _REPO_LIST:
-		with urlopen(r) as remote:
-			big_list += "\n" + remote.read().decode("UTF-8")
+	for r in _REPO_LIST + get_external_repos():
+		try:
+			with urlopen(r) as remote:
+				big_list += "\n" + remote.read().decode("UTF-8")
+		except:
+			print("WARN: Repo unavailable: " + r)
 
 	# Remove unnecessary newlines, split into array
 	big_list = re.sub(r"^(?:[\t ]*(?:\r?\n|\r))+", "", big_list)
@@ -55,3 +68,23 @@ def fetch_ignore(name):
 
 	with urlopen(_sources[name]) as remote:
 		return remote.read().decode("UTF-8")
+
+
+def add_source(url):
+	r = get_external_repos()
+
+	if url not in r:
+		r.append(url)
+
+	config.store("repos", r)
+
+
+def remove_source(url):
+	r = get_external_repos()
+
+	if url in r:
+		r.remove(url)
+		config.store("repos", r)
+		return True
+
+	return False
